@@ -10,7 +10,7 @@ import os
 import logzero
 from logzero import logger
 import time
-
+import ssl
 from SmartApi.version import __version__, __title__
 
 log = logging.getLogger(__name__)
@@ -30,6 +30,7 @@ class SmartConnect(object):
         "api.user.profile": "/rest/secure/angelbroking/user/v1/getProfile",
 
         "api.order.place": "/rest/secure/angelbroking/order/v1/placeOrder",
+        "api.order.placefullresponse": "/rest/secure/angelbroking/order/v1/placeOrder",
         "api.order.modify": "/rest/secure/angelbroking/order/v1/modifyOrder",
         "api.order.cancel": "/rest/secure/angelbroking/order/v1/cancelOrder",
         "api.order.book":"/rest/secure/angelbroking/order/v1/getOrderBook",
@@ -92,12 +93,34 @@ class SmartConnect(object):
         self.accept=self.accept
         self.userType=self.userType
         self.sourceID=self.sourceID
+
+        # Create SSL context
+        self.ssl_context = ssl.create_default_context()
+        self.ssl_context.options |= ssl.OP_NO_TLSv1  # Disable TLS 1.0
+        self.ssl_context.options |= ssl.OP_NO_TLSv1_1  # Disable TLS 1.1
+
+        # Configure minimum TLS version to TLS 1.2
+        self.ssl_context.minimum_version = ssl.TLSVersion.TLSv1_2
+
+        if not disable_ssl:
+            self.reqsession = requests.Session()
+            if pool is not None:
+                reqadapter = requests.adapters.HTTPAdapter(**pool)
+                self.reqsession.mount("https://", reqadapter)
+            else:
+                reqadapter = requests.adapters.HTTPAdapter()
+                self.reqsession.mount("https://", reqadapter)
+            logger.info(f"in pool")
+        else:
+            # If SSL is disabled, use the default SSL context
+            self.reqsession = requests
+            
         # Create a log folder based on the current date
         log_folder = time.strftime("%Y-%m-%d", time.localtime())
         log_folder_path = os.path.join("logs", log_folder)  # Construct the full path to the log folder
         os.makedirs(log_folder_path, exist_ok=True) # Create the log folder if it doesn't exist
         log_path = os.path.join(log_folder_path, "app.log") # Construct the full path to the log file
-        logzero.logfile(log_path, loglevel=logging.INFO)  # Output logs to a date-wise log file
+        logzero.logfile(log_path, loglevel=logging.ERROR)  # Output logs to a date-wise log file
 
         if pool:
             self.reqsession = requests.Session()
@@ -184,6 +207,7 @@ class SmartConnect(object):
                                         proxies=self.proxies)
            
         except Exception as e:
+            logger.error(f"Error occurred while making a {method} request to {url}. Headers: {headers}, Request: {params}, Response: {e}")
             raise e
 
         if self.debug:
@@ -207,7 +231,8 @@ class SmartConnect(object):
                 # native errors
                 exp = getattr(ex, data["error_type"], ex.GeneralException)
                 raise exp(data["message"], code=r.status_code)
-
+            if data.get("status",False) is False : 
+                logger.error(f"Error occurred while making a {method} request to {url}. Error: {data['message']}. URL: {url}, Headers: {headers}, Request: {params}, Response: {data}")
             return data
         elif "csv" in headers["Content-type"]:
             return r.content
@@ -286,18 +311,38 @@ class SmartConnect(object):
     def getProfile(self,refreshToken):
         user=self._getRequest("api.user.profile",{"refreshToken":refreshToken})
         return user
-    
-    def placeOrder(self,orderparams):
 
+    def placeOrder(self,orderparams):
         params=orderparams
-       
         for k in list(params.keys()):
             if params[k] is None :
                 del(params[k])
-        
-        orderResponse= self._postRequest("api.order.place", params)['data']['orderid']
-    
-        return orderResponse
+        response= self._postRequest("api.order.place", params)
+        if response is not None and response.get('status', False):
+            if 'data' in response and response['data'] is not None and 'orderid' in response['data']:
+                orderResponse = response['data']['orderid']
+                return orderResponse
+            else:
+                logger.error(f"Invalid response format: {response}")
+        else:
+            logger.error(f"API request failed: {response}")
+        return None
+
+    def placeOrderFullResponse(self,orderparams):
+        params=orderparams
+        for k in list(params.keys()):
+            if params[k] is None :
+                del(params[k])
+        response= self._postRequest("api.order.placefullresponse", params)
+        if response is not None and response.get('status', False):
+            if 'data' in response and response['data'] is not None and 'orderid' in response['data']:
+                orderResponse = response
+                return orderResponse
+            else:
+                logger.error(f"Invalid response format: {response}")
+        else:
+            logger.error(f"API request failed: {response}")
+        return None
     
     def modifyOrder(self,orderparams):
         params = orderparams
